# Generated by Django 3.2.4 on 2021-07-15 07:48

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pretixbase", "0195_auto_20210622_1457"),
        ("pretix_vacc_autosched", "0003_itemconfig_max_days"),
    ]

    operations = [
        migrations.AddField(
            model_name="itemconfig",
            name="second_item",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="pretixbase.item",
            ),
        ),
    ]
