# Generated by Django 3.0.12 on 2021-03-10 12:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pretixbase", "0175_orderrefund_comment"),
        ("pretix_vacc_autosched", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="itemconfig",
            name="item",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="vacc_autosched_config",
                to="pretixbase.Item",
            ),
        ),
        migrations.CreateModel(
            name="LinkedOrderPosition",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                (
                    "base_position",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="vacc_autosched_linked",
                        to="pretixbase.OrderPosition",
                    ),
                ),
                (
                    "child_position",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="vacc_autosched_link",
                        to="pretixbase.OrderPosition",
                    ),
                ),
            ],
        ),
    ]
